// Class automatically generated by Dev-C++ New Class wizard

#include <sys/types.h>
#include <math.h>
#include <dirent.h>
#include "mesh.h"
#include "piece.h"
#include "connect4.h" // class's header file


#ifdef DMALLOC
#include <dmalloc.h>
#endif

#define ABINF 84765625

void multTransform(float *t1,float *t2)
{
	int i,j,k;
	float r[16];
	for(i=0;i<16;i++) r[i]=0;
	for(i=0;i<4;i++) {
		for(j=0;j<4;j++) {
			for(k=0;k<4;k++) {
				r[i*4+j]+=t1[i*4+k]*t2[k*4+j];
			}
		}
	}
	for(i=0;i<16;i+=4) {
		printf("[%.2f %.2f %.2f %.2f]x",t1[i+0],t1[i+1],t1[i+2],t1[i+3]);
		printf("[%.2f %.2f %.2f %.2f]=",t2[i+0],t2[i+1],t2[i+2],t2[i+3]);
		printf("[%.2f %.2f %.2f %.2f]\n",r[i+0],r[i+1],r[i+2],r[i+3]);
	}
	printf("\n");
	for(i=0;i<16;i++) t1[i]=r[i];
}

void rotateTransformXYZ(float *transform,float xrot,float yrot,float zrot)
{
	float t[16];
	int i;
	int from[]={1,0,0};
	int to[]={2,2,1};
	float angle[3];
	int dir;

	angle[0]=xrot;
	angle[1]=yrot;
	angle[2]=zrot;

	for(dir=0;dir<3;dir++) {
		for(i=0;i<16;i++) t[i]=(i%4)==i/4?1:0;
		float c=cosf(angle[dir]);
		float s=sinf(angle[dir]);
		t[from[dir]*4+from[dir]]=c;
		t[to[dir]*4+from[dir]]=-s;
		t[from[dir]*4+to[dir]]=s;
		t[to[dir]*4+to[dir]]=c;

		multTransform(transform,t);
	}
}

void scaleTransform(float *transform,float x,float y,float z)
{
	float t[16];
	int i;

	for(i=0;i<16;i++) t[i]=(i%4)==i/4?1:0;
	t[0]=x;
	t[5]=y;
	t[10]=z;
	multTransform(transform,t);
}

void translateTransform(float *transform,float x,float y,float z)
{
	float t[16];
	int i;

	for(i=0;i<16;i++) t[i]=(i%4)==i/4?1:0;
	t[12]=x;
	t[13]=y;
	t[14]=z;
	multTransform(transform,t);
}

// class constructor
Connect4::Connect4()
{
	score=0;
	lastTick=0;
	sceneEntity=0;
	tokenPiece=0;
	arrowPiece=0;
	modelBoard=0;
	modelRed=0;
	modelYellow=0;
	modelArrow=0;
	modelRoom=0;
	themeNo=0;
	sound=true;
	pauseMenu=false;
	mode=GM_MODEMENU;
}

// class destructor
Connect4::~Connect4()
{
	std::vector<Piece *>::iterator p;
	for(p=piece.begin();p!=piece.end();p++) {
		Piece *pp=*p;
		delete pp;
	}
	piece.clear();
	if(modelBoard) free_entity(modelBoard);
	modelBoard=0;
	if(modelRed) free_entity(modelRed);
	modelRed=0;
	if(modelYellow) free_entity(modelYellow);
	modelYellow=0;
	if(modelArrow) free_entity(modelArrow);
	modelArrow=0;
	if(sceneEntity) delete sceneEntity;
	sceneEntity=0;
}

void Connect4::switchTheme()
{
    Piece *pp=0; 
	if(theme.size()<1) return;
	if(modelRoom) free_entity(modelRoom);

	std::vector<Piece *>::iterator p;
	for(p=piece.begin();p!=piece.end();p++) {
		pp=*p;
		if(pp->type==PT_ROOM) break;
	}
	if(pp && pp->type!=PT_ROOM) pp=0;

	themeNo++;
	if(themeNo>=theme.size()) themeNo=0;
	modelRoom=load_xof(theme[themeNo].c_str());
	if(pp) pp->setModel(modelRoom,PT_ROOM);
}

void Connect4::orientPiece(Entity *modelPiece)
{
	if(modelPiece) {
		std::vector<Frame *>::iterator p;
		for(p=modelPiece->frame.begin();p!=modelPiece->frame.end();p++) {
			Frame *f;
			f=*p;
			float *t=f->transform;
			translateTransform(t,-t[12],-t[13],-t[14]);
			scaleTransform(t,0.105,0.105,-0.105);
			rotateTransformXYZ(t,M_PI/2,M_PI/2,0);
			scaleTransform(t,1,1,-1);
		}
	}
}

/*
 * Loads in the level specified, and initializes all things needed.
 * 
 * @level the level number to load (0 based)
 * @returns 0 on success, negative on error.
 */
int Connect4::load(int level)
{
	if(!modelRoom) {
		modelRoom=load_xof("models/background.x");
		//orientPiece(modelRoom);
	}
	if(!modelBoard) modelBoard=load_xof("models/board4.x.gz");
	if(!modelRed) {
		modelRed=load_xof("models/redpiece.x.gz");
		orientPiece(modelRed);
	}
	if(!modelYellow) {
		modelYellow=load_xof("models/yellowpiece.x.gz");
		orientPiece(modelYellow);
	}
	if(!modelArrow) {
		modelArrow=load_xof("models/arrow.x.gz");
		orientPiece(modelArrow);
	}
	//if(!sceneEntity) sceneEntity=load_xof("map/market1.x");
	std::vector<Piece *>::iterator p;
	for(p=piece.begin();p!=piece.end();p++) {
		Piece *pp=*p;
		delete pp;
	}
	piece.clear();
	Piece *pp=new Piece();
	pp->setModel(modelRoom,PT_ROOM);
	piece.push_back(pp);
#if 1
	scaleTransform(pp->getTransform(),0.108f,0.108f,-0.108f);
	rotateTransformXYZ(pp->getTransform(),M_PI/2,M_PI/2,0);
	scaleTransform(pp->getTransform(),1,1,-1);
	translateTransform(pp->getTransform(),40,-43,0);
#else
	pp->getTransform()[0]=0.0f;
	pp->getTransform()[2]=0.108f;
	pp->getTransform()[5]=0.0f;
	pp->getTransform()[4]=0.108f;
	pp->getTransform()[9]=0.108f;
	pp->getTransform()[10]=0.0f;
	pp->getTransform()[12]=10;
	pp->getTransform()[13]=-43;
	pp->getTransform()[14]=-20;
#endif
	pp=new Piece();
#if 1
	pp->setModel(modelBoard,PT_BOARD);
	piece.push_back(pp);
	pp->getTransform()[0]=0.108f;
	pp->getTransform()[5]=0.108f;
	pp->getTransform()[10]=0.108f;
#endif
#if 1
	activePlayer=BP_RED;
	pp=new Piece();
	pp->setModel(modelRed,PT_RED);
	piece.push_back(pp);
	float x,y,z;
	calcPiecePos(10,-1,x,y,z);
	pp->setPos(x,y,z);
	calcPiecePos(-3,-1,x,y,z);
	pp->setMoveTo(x,y,z,1);
	tokenPiece=pp;
#endif
	int i,j;
#if 1
	pp=new Piece();
	pp->setModel(modelArrow,PT_ARROW);
	piece.push_back(pp);
/*
	pp->getTransform()[4+2]=0.5f;
	pp->getTransform()[4+1]=0.0f;
	pp->getTransform()[0+0]=0.5f;
	pp->getTransform()[8+2]=0.0f;
	pp->getTransform()[8+1]=-0.5f;
*/
	//printf("Calling moveto\n");
	{
		float x,y,z;
		calcPiecePos(-3,-3,x,y,z);
		pp->setPos(x,y,z);
		calcPiecePos(0,-1,x,y,z);
		pp->setMoveTo(x,y,z,1);
	}
	arrowPiece=pp;
	arrowColumn=0;
#endif
	for(j=0;j<6;j++) {
		for(i=0;i<7;i++) {
			bp[j][i]=BP_EMPTY;
		}
	}

	// Not the right spot
	//mode=GM_PVSP;
	//mode=GM_PVSAI;
	
	if(theme.size()<1) {
        FILE *file=fopen("themes.txt","r");
        if(file) {
            char line[256];
            while( fgets(line,255,file)) {
                line[255]=0;
                char *s=line;
                while(s[0]==' ') s++;	// Remove leading spaces
                if(strchr(s,'\n')) strchr(s,'\n')[0]=0;	// Remove trailing CR/LF
                if(strchr(s,'\r')) strchr(s,'\r')[0]=0;	// Remove trailing CR/LF
                if(strlen(s)>3) {
                    theme.push_back((std::string)s);
                }
            }
            fclose(file);
        }
    }

	playSound(S_LEVELSTART);
	return 0;
}


// returns the value of sceneEntity
Entity * Connect4::getSceneEntity()
{
	return sceneEntity;
}

// returns the value of score
int Connect4::getScore()
{
	return score;
}

/*
 * New game is called to initialize the game engine.  It resets
 * the score, health, level, loads the initial level, and resets 
 * the NPCs.
 */
void Connect4::newGame()
{
	score=0;
	lastTick=0;
	//load(1);
}

/*
 * Updates the state of all entities in the currently active level.
 * 
 * @param milliseconds milliseconds since the game/level/machine started.
 */
void Connect4::updateLevel(unsigned long milliseconds)
{
	std::vector<Piece *>::iterator p;
	if(lastTick==0) lastTick=milliseconds-1;
	for(p=piece.begin();p!=piece.end();p++) {
		Piece *bot;
		bot=*p;
		//printf("Calling update.\n");
		bot->update(milliseconds-lastTick);
	}
	if(activePlayer!=BP_EMPTY && winCondition()) {
		enum BoardPiece player=winCondition();
		for(p=piece.begin();p!=piece.end();p++) {
			Piece *pp=*p;
			if(pp->action!=PA_NONE) return;	// Wait for anim
		}
		activePlayer=BP_EMPTY;
		// Show winner animation.
		playSound(mode!=GM_PVSAI?S_WINNER:player==BP_YELLOW?S_WINNER:S_COMPUTERWINNER);
		if(player==BP_TIE) mode=GM_TIE;
		if(player==BP_YELLOW) mode=(mode!=GM_PVSAI)?GM_YELLOWWINS:GM_CPUWINS;
		if(player==BP_RED) mode=GM_REDWINS;
	}
	// Here is where the game ai goes.
	bool aiActive=false;
	if(activePlayer==BP_YELLOW && mode==GM_PVSAI) {
		aiActive=true;
		for(p=piece.begin();p!=piece.end();p++) {
			Piece *pp=*p;
			if(pp->action!=PA_NONE) {
				aiActive=false;	// Wait for anim
				break;
			}
		}
	}
	if(aiActive) {
		// Call the AI to make it's move.
#ifdef _DEBUG
		printf("Searching for move for yellow\n");
#endif
		int column=searchForMove(BP_YELLOW);
		if(column==-1) {
			printf("Internal error.\n");
		} else {
			dropPiece(column);
		}
#ifdef _DEBUG
		int i,j;
		for(j=0;j<6;j++) {
			for(i=0;i<7;i++) {
				printf(" %c",bp[j][i]==BP_EMPTY?'.':bp[j][i]==BP_RED?'X':bp[j][i]==BP_YELLOW?'O':bp[j][i]==BP_REDPROPOSED?'x':bp[j][i]==BP_YELLOWPROPOSED?'o':'?'); 
			}
			printf("\n");
		}
#endif
	}
	
	// Over.
	lastTick=milliseconds;
}

void Connect4::dropPiece(int arrowColumn)
{
	float x,y,z;
	Piece *pp=new Piece();
	pp->setModel(activePlayer==BP_RED?modelRed:modelYellow,activePlayer==BP_RED?PT_RED:PT_YELLOW);
	calcPiecePos(arrowColumn,-4,x,y,z);
	pp->setPos(x,y,z);
	int row=getRow(arrowColumn);
	calcPiecePos(arrowColumn,row,x,y,z);
	pp->setMoveTo(x,y,z,2.0f);
	piece.push_back(pp);
#ifdef _DEBUG
	printf("Added piece to %d,%d\n",arrowColumn,row);
	printf("Now there are %d pieces.\n",piece.size());
#endif
	playSound(S_DROPPIECE);
	bp[row][arrowColumn]=activePlayer;
	activePlayer=activePlayer==BP_RED?BP_YELLOW:BP_RED;
	if(activePlayer==BP_RED) {
		tokenPiece->setModel(modelRed,PT_RED);
	} else {
		tokenPiece->setModel(modelYellow,PT_YELLOW);
	}
#ifdef _DEBUG
	printf("Player %s turn\n",activePlayer==BP_RED?"red":"yellow");
#endif
}

/*
 * Receives a button event.
 * 
 * @param id the button id of the button (0=triangle, etc).
 * @param down true if the button is pressed, and false if the button was released.
 */
void Connect4::buttonEvent(int id,int down,int milliseconds)
{
	if(pauseMenu) {

		if(down==0 && id==FF_UP) { arrowColumn--; if(arrowColumn<0) arrowColumn=0; else playSound(S_MOVEARROW); }
		if(down==0 && id==FF_DOWN) { arrowColumn++; if(arrowColumn>4) arrowColumn=4; else playSound(S_MOVEARROW); }
		if(down==0 && (id==FF_CROSS || id==FF_START)) {
			if(arrowColumn==1) { sound=!sound; return; }
			if(arrowColumn==2) mode=GM_MODEMENU;
			if(arrowColumn==3) mode=GM_QUIT;
			if(arrowColumn==4) {
               switchTheme();
               return;
            }
			//if(arrowColumn==0) { pauseMenu=false; return; }
			pauseMenu=false;
		}
		return;
	}
	if(mode==GM_MODEMENU) {
		if(arrowColumn>4) arrowColumn=0;
		if(down==0 && id==FF_UP) { arrowColumn--; if(arrowColumn<0) arrowColumn=0; else playSound(S_MOVEARROW); }
		if(down==0 && id==FF_DOWN) { arrowColumn++; if(arrowColumn>4) arrowColumn=4; else playSound(S_MOVEARROW); }
		if(down==0 && id==FF_CROSS) {
			if(arrowColumn==0) mode=GM_PVSP;
			if(arrowColumn==1) mode=GM_PVSAI;
			if(arrowColumn==2) mode=GM_SERVER;
			if(arrowColumn==3) mode=GM_CLIENT;
			if(arrowColumn==4) mode=GM_QUIT;
			load(1);
		}
		return;
	}
	if(down==0 && id==FF_START) {
		pauseMenu=true;
		arrowColumn=0;
		return;
	}
	if(activePlayer==BP_EMPTY) {
		// We've declared a winner.
#ifdef _DEBUG
		printf("Restarting game\n");
#endif
		if(down==0 && (id==FF_DOWN || id==FF_CROSS)) {
			//load(1);
			mode=GM_MODEMENU;
			arrowColumn=0;
		}
	}
	if(down && (id==FF_LEFT || id==FF_RIGHT || id==FF_SQUARE || id==FF_CIRCLE) && arrowPiece->action==PA_NONE) {	// Move the arrow to the right spot.
#ifdef _DEBUG
		printf("Handling arrows\n");
#endif
		if(activePlayer==BP_YELLOW && mode==GM_PVSAI) return;
		if((id==FF_LEFT && activePlayer==BP_RED) || (id==FF_SQUARE && activePlayer==BP_YELLOW)) {
			if(arrowColumn>0) {
#ifdef _DEBUG
				printf("Handling left\n");
#endif
				arrowColumn--;
				float x,y,z;
				calcPiecePos(arrowColumn,-1,x,y,z);
				arrowPiece->setMoveTo(x,y,z,0.5f);
				playSound(S_MOVEARROW);
			}
		}
		if((id==FF_RIGHT && activePlayer==BP_RED) || (id==FF_CIRCLE && activePlayer==BP_YELLOW)) {
			if(arrowColumn<6) {
#ifdef _DEBUG
				printf("Handling right\n");
#endif
				arrowColumn++;
				float x,y,z;
				calcPiecePos(arrowColumn,-1,x,y,z);
				arrowPiece->setMoveTo(x,y,z,0.5f);
				playSound(S_MOVEARROW);
			}
		}
	}
	if(down && (id==FF_DOWN || id==FF_CROSS) && arrowPiece->action==PA_NONE) {	// Deposit a piece on the board
		if(activePlayer==BP_YELLOW && mode==GM_PVSAI) return;
		if(id==FF_DOWN && activePlayer!=BP_RED) return;
		if(id==FF_CROSS && activePlayer!=BP_YELLOW) return;
#ifdef _DEBUG
		printf("Handling down\n");
#endif
		int row=getRow(arrowColumn);
		if(row==-1) {
			playSound(S_BADMOVE);
			return;
		}

		dropPiece(arrowColumn);
	}
}

/*
 * A keyboard event was received.
 * 
 * @param ch ASCII character or special keycode.
 * @param down true if pressed, or false if released.
 */
void Connect4::keyEvent(int ch,int down)
{
	/* TODO (#1#): Implement Connect4::keyEvent() */
}

void Connect4::clearProposals()
{
	int i,j;
	for(j=0;j<6;j++) {
		for(i=0;i<7;i++) {
			if(bp[j][i]==BP_REDPROPOSED || bp[j][i]==BP_YELLOWPROPOSED) bp[j][i]=BP_EMPTY;
		}
	}
}

int Connect4::getRow(int column)
{
	int row;
	if(column<0 || column>6) return -1;

	for(row=5;row>=0;row--) {
		if(bp[row][column]==BP_EMPTY) return row;
	}
	return -1;
}

int Connect4::searchForMove(enum BoardPiece player)
{
	int alpha=-ABINF,beta=ABINF,bestColumn=-1;
#ifdef _DEBUG
	printf("Starting AI search\n");
#endif
	searchForMove(player,5,alpha,beta,bestColumn);
#ifdef _DEBUG
	printf("Ended AI search\n");
#endif
	return bestColumn;
}

int Connect4::searchForMove(enum BoardPiece player,int maxDepth,int alpha,int beta,int &bestColumn)
{
	// Try each of the 7 columns.
	bestColumn=-1;
	int column;
	int worth=0;

#if 0
	printf("Input alpha=%d,beta=%d\n",alpha,beta);
#endif

	enum BoardPiece winner=winCondition();
	if(maxDepth==0 || winner!=BP_EMPTY) {
		int worth=positionPotiential(activePlayer);
#ifdef _DEBUG
		printf("leaf node: player=%s; depth=%d; worth=%d\n",player==BP_RED?"red":"yellow",maxDepth,worth);
#endif
		return worth;
	}
	if(player!=activePlayer) {
		// Could re-order based on iterative deepening priors.
		for(column=0;column<7;column++) {	
			int row=getRow(column);
			if(row==-1) continue;
			// Now let's propose that, and see how well we do.
#ifdef _DEBUG
			printf("%d Test Move: %d,%d for player %s\n",maxDepth,column,row,player==BP_RED?"red":"yellow");
#endif
			bp[row][column]=player==BP_RED?BP_REDPROPOSED:BP_YELLOWPROPOSED;
			// Should do bredth first with iterative deepening.
			worth=0;
			int subBeta;
			int subColumn=-1;

			subBeta=searchForMove(player==BP_RED?BP_YELLOW:BP_RED,maxDepth-1,alpha,beta,subColumn);
			if(subBeta<=beta) {	// MIN
#ifdef _DEBUG
				printf("Updating beta from %d to %d\n",beta,subBeta);
#endif
				beta=subBeta;
				bestColumn=column;
			}
			bp[row][column]=BP_EMPTY;
			if(alpha>=beta) {
#ifdef _DEBUG
				printf("Pruning alpha: alpha %d >= beta %d\n",alpha,beta);
#endif
				return alpha;
			}
		}
#ifdef _DEBUG
		printf("Returning beta: alpha %d. beta %d\n",alpha,beta);
#endif
		return beta;
	} else {
		// Could re-order based on iterative deepening priors.
		for(column=0;column<7;column++) {	
			int row=getRow(column);
			if(row==-1) continue;
			// Now let's propose that, and see how well we do.
#ifdef _DEBUG
			printf("%d Test Move: %d,%d for player %s\n",maxDepth,column,row,player==BP_RED?"red":"yellow");
#endif
			bp[row][column]=player==BP_RED?BP_REDPROPOSED:BP_YELLOWPROPOSED;
			// Should do bredth first with iterative deepening.
			worth=0;
			int subAlpha;
			int subColumn=-1;

			subAlpha=searchForMove(player==BP_RED?BP_YELLOW:BP_RED,maxDepth-1,alpha,beta,subColumn);
			if(subAlpha>=alpha) {	// MAX
#ifdef _DEBUG
				printf("Updating alpha from %d to %d\n",alpha,subAlpha);
#endif
				alpha=subAlpha;
				bestColumn=column;
			}
			bp[row][column]=BP_EMPTY;
			if(alpha>=beta) {
#ifdef _DEBUG
				printf("Pruning beta: alpha %d >= beta %d\n",alpha,beta);
#endif
				return beta;
			}
		}
	}
#ifdef _DEBUG
	printf("Returning alpha: alpha %d. beta %d\n",alpha,beta);
#endif
	return alpha;
}

int Connect4::isPlayer(enum BoardPiece player,int x,int y)
{
	if(x<0 || x>=7 || y<0 || y>=6) return 0;
	if(bp[y][x]==player || bp[y][x]==(player==BP_RED?BP_REDPROPOSED:BP_YELLOWPROPOSED)) return 10;  // Did go here.
	if(x==getRow(y) && bp[y][x]==BP_EMPTY) return 2;	// Could go here next
	if(bp[y][x]==BP_EMPTY) return 1;	// Could go here
	return 0;	// Can't go here.
}

int Connect4::potientialFour(enum BoardPiece player,int x,int y,int dx,int dy)
{
	// Check for BT_EMPTY or player pieces in every possible 4 in a row in -dx,-dy and dx,dy directions.
	int worth=0;
	int i;
	int o;

	for(o=-3;o<=0;o++) {
		int w=1;
		for(i=0;i<4;i++) {
			int res=isPlayer(player,x+(o+i)*dx,y+(o+i)*dy);
			//printf("%d*",res);
			if(res==0) {
				w=0;
				break;
			}
			w+=res;
		}
		if(w>=20) {
			w=ABINF;
		} else if(w>=15) {
			w=6875;
		} else if(w>=10) {
			w=45;
		} else if(w>=5) {
			w=5;
		} else {
		}
		//printf("=%d\n",w);
		worth+=w;
	}
	// Now scale up the worth, depending on the context.
	
	return worth;	// Max of 64 (3 on either side), min (winable) 4, min of 0.
}

int Connect4::positionPotiential(enum BoardPiece player)
{
	// Look for win combos, and rate the value of the board.
	int column;
	int worth=0;

	enum BoardPiece winner=winCondition();
	if(winner==BP_RED) return player==BP_RED?ABINF:-ABINF;
	if(winner==BP_YELLOW) return player==BP_YELLOW?ABINF:-ABINF;
	if(winner==BP_TIE) return 0;	// Noone wins.
	int plus=0,minus=0;
	enum BoardPiece opponent=player==BP_RED?BP_YELLOW:BP_RED;
	
	for(column=0;column<7;column++) {
		int row=getRow(column);
		// Try to construct 4 in a row in various ways.
		if(row==-1) continue;	// Can't add anything to this row.
		// Think about combinations of 4 if we add a player piece to this column.
		plus+=potientialFour(player,row,column,0,1);
		plus+=potientialFour(player,row,column,1,0);
		plus+=potientialFour(player,row,column,1,1);
		plus+=potientialFour(player,row,column,-1,1);
		

		// opponent's potiential
		minus-=potientialFour(opponent,row,column,0,1);
		minus-=potientialFour(opponent,row,column,1,0);
		minus-=potientialFour(opponent,row,column,1,1);
		minus-=potientialFour(opponent,row,column,-1,1);
	}
	worth=plus+minus/2;
	//printf("plus: %d, minus: %d; worth=%d\n",plus,minus,worth);
	return worth;
}

enum BoardPiece Connect4::winCondition()
{
	int i,j,d,o,w;
	int empty=0;

	int dx[4]={0,1,1,1};
	int dy[4]={1,0,1,-1};
	// find 4 in a rows.
	for(j=0;j<6;j++) {
		for(i=0;i<7;i++) {
			enum BoardPiece player;
			if(bp[j][i]==BP_EMPTY) {
				empty++;
				continue;
			}
			player=bp[j][i];
			if(player==BP_REDPROPOSED) player=BP_RED;
			if(player==BP_YELLOWPROPOSED) player=BP_YELLOW;
			// Use the piece, and look every way
			for(d=0;d<4;d++) {
				// Grow a solution
				w=5;	// Starting point.
				for(o=1;o<4;o++) {
					int res=isPlayer(player,i+o*dx[d],j+o*dy[d]);
					if(res==0 || res==1 || res==2) {
						w=0;
						break;
					}
					//printf("win?: %d,%d (%d:+%d+%d) %d=%d\n",i,j,d,dx[d],dy[d],o,w);
					w+=res;
				}
				if(w>0) return player;
			}
		}
	}
	if(empty==0) {
		return BP_TIE;
	}
	return BP_EMPTY;
}

void Connect4::calcPiecePos(int column,int row,float &x,float &y,float &z)
{
	x=-0.4;
	y=-row*3.0f;
	z=-1.5f-column*3.8f;
	//printf("Row: %d, Column %d => %.2f,%.2f,%.2f\n",row,column,x,y,z);
}
