#ifdef WIN32
#include<windows.h>
#include<mmsystem.h>	// timeGetTime()
#endif

//#define SURVIVALHORROR 0

#include <stdlib.h>
#include <string.h>
#include <GL/glut.h>
#include <GL/glu.h>
#include <GL/gl.h>
#ifndef WIN32
#include <sys/time.h>
#endif

#include<vector>
#include<map>
#include<string>

#include "glchk.h"
#include "mesh.h"
#include "loadpng.h"
#include "connect4.h"

#ifdef DMALLOC
#include "dmalloc.h"
#endif

#ifndef WIN32
unsigned long timeGetTime()
{
	struct timeval tv;
	struct timezone tz;

	gettimeofday(&tv,&tz);
	//printf("time is: %u - %u\n",(unsigned int)tv.tv_sec,(unsigned int)tv.tv_usec);

	unsigned long result=(long long)tv.tv_sec*1000+(long long)tv.tv_usec/1000;
	//printf("Result: %u\n",result);
	return result;
}
#endif

int viewmode=5;

static
void reshape (int w, int h)
{
	GLCHK(glViewport(0, 0, w, h));
	GLCHK(glMatrixMode(GL_PROJECTION));
	GLCHK(glLoadIdentity());
//	GLCHK(glOrtho(-20, 20, -20, 20, -200, 200));
//	GLCHK(glFrustum(-20.0,20.0,-20.0,20.0,1.5,40.0));
	GLCHK(gluPerspective(45.0,(float)w/h,1.5,512.0));
	GLCHK(glMatrixMode(GL_MODELVIEW));
	GLCHK(glLoadIdentity());
}


static float delta = 1.0;
#define NTEX 15
GLuint texid[NTEX];
int nextTex;
std::map<std::string,int> texMap;

int lastTex=-1;

void display_mesh(Mesh *mesh)
{
	int i,j;
	if(!mesh) return;
	if(mesh->mat.size()>0 && mesh->mat.begin()->textureName.size()>0) {
		std::string name=mesh->mat.begin()->textureName;
		if(texMap.find(name)!=texMap.end()) {
			int id=texMap[name];
			Image *image=mesh->mat.begin()->texture;
			if(id!=lastTex) {
				//printf("Binding texture for image %s\n",name.c_str());
				lastTex=id;
				GLCHK(glBindTexture(GL_TEXTURE_2D, id));
				GLCHK(glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, image->textureWidth, image->textureHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, image->data));
				GLCHK(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR));
				GLCHK(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR));
			}
		} else {
			return;
		}
	}
//printf("Displaying mesh %08x (%d poly, %d verts)\n",mesh,mesh->polyCount,mesh->vertCount);
	glBegin(GL_TRIANGLES);
		for(i=0;i<mesh->polyCount;i++) {
			for(j=0;j<3;j++) {
				int vt=mesh->poly[i].v[j];
				struct Vertex3D *vert=mesh->vert+vt;
				glColor3f(1, 1, 1);
				glTexCoord2f(vert->u,vert->v);
				glNormal3f(vert->nx,vert->ny,vert->nz);
				glVertex3f(vert->x, vert->y, vert->z);
			}
		}
	GLCHK(glEnd());
}

int frameDepth=0;

void display_frame(Frame *frame)
{
	if(!frame) return;
	std::vector<Frame *>::iterator p;
	std::vector<Mesh *>::iterator q;
	GLCHK(glMatrixMode(GL_MODELVIEW));
	GLCHK(glPushMatrix());
	GLCHK(glMultMatrixf(frame->transform));
	//printf("displaying Frame: %s\n",frame->name.c_str());
#if 0
	int i,j;
	for(i=0;i<16;i+=4) {
		for(j=0;j<frameDepth;j++) printf(">");
		printf("> [%.2f %.2f %.2f %.2f]\n",frame->transform[i+0],frame->transform[i+1],frame->transform[i+2],frame->transform[i+3]);
	}
#endif
//	int i=0;
	frameDepth++;
	for(p=frame->frame.begin();p!=frame->frame.end();p++) {
		Frame *frame=*p;
		display_frame(frame);
	}
	frameDepth--;
	for(q=frame->mesh.begin();q!=frame->mesh.end();q++) {
		Mesh *mesh=*q;
		
		display_mesh(mesh);
		/*
		char buf[16];
		sprintf(buf,"%d.tmsh",i++);
		export_tmsh6((frame->name+buf).c_str(),mesh);
		*/
	}

	GLCHK(glPopMatrix());
}

void display_entity(Entity *entity)
{
        std::vector<Frame *>::iterator p;

	if(!entity) return;

	for(p=entity->frame.begin();p!=entity->frame.end();p++) {
		Frame *frame=*p;
		display_frame(frame);
	}

}

void upload_textures(TextureHolder *entity)
{
	std::map<std::string,Image *>::iterator p;
	
	printf ("Uploading textureholder %08x textures\n",entity);
	if(!entity) return;
	for(p=entity->textureBegin();p!=entity->textureEnd();p++) {
		std::string name=p->first;
		Image *image=p->second;
		printf ("Uploading %dx%d texture\n",image->imageWidth,image->imageHeight);
		if(nextTex<NTEX) {
			GLCHK(glBindTexture(GL_TEXTURE_2D, texid[nextTex]));
			psp_log("uploading texture data for %d\n", texid[nextTex]);
			GLCHK(glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, image->textureWidth, image->textureHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, image->data));
			texMap[name]=texid[nextTex];
			nextTex++;
		}
	}
}

static int freeze=0;
static int step=0;
static float cameraz=-16,zoom=1.0;

Connect4 fishFight;

static
void display (void)
{
	int i,j;
	static GLfloat angle;

	// Display the 2d background
	GLCHK(glClear(GL_COLOR_BUFFER_BIT));
	GLCHK(glClear(GL_DEPTH_BUFFER_BIT));

#ifdef SURVIVALHORROR
	std::string bgFilename;
	Image *bg=0;
	Viewpoint *camera=fishFight.getCamera();
	if(camera) {
		bgFilename=camera->bgFilename;
		bg=camera->bg;
		printf("Displaying bg: %s (%08x)\n",bgFilename.c_str(),camera->bg);
//		bgFilename=vp->bgFilename;		
	}
	
#if 0
	if(bgFilename.size()>0) {
		if(texMap.find(bgFilename)!=texMap.end()) {
			int id=texMap[bgFilename];
			Image *image=fishFight.findTexture(bgFilename.c_str());
			//printf("Found Image %08x (%08x is correct)\n",image,bg);
			if(id!=lastTex) {
//				printf("Binding texture for image %s\n",name.c_str());
				lastTex=id;
				GLCHK(glBindTexture(GL_TEXTURE_2D, id));
				GLCHK(glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, image->textureWidth, image->textureHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, image->data));
				GLCHK(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR));
				GLCHK(glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR));
			}
			GLCHK(glMatrixMode(GL_MODELVIEW));
			GLCHK(glLoadIdentity());
			GLCHK(glBegin(GL_QUADS));
			(glTexCoord2f(0,0));
			(glVertex3f(-20,-20,0));
			(glTexCoord2f(1,0));
			(glVertex3f(20,-20,0));
			(glTexCoord2f(1,1));
			(glVertex3f(20,20,0));
			(glTexCoord2f(0,1));
			(glVertex3f(-20,20,0));
			GLCHK(glEnd());
		}
	}
#endif
#endif

	// Now 3d time
	GLCHK(glMatrixMode(GL_MODELVIEW));
	GLCHK(glLoadIdentity());

#ifdef SURVIVALHORROR
	if(camera) {
			GLCHK(gluLookAt(camera->transform[12],camera->transform[13],camera->transform[14],
			camera->at[0],camera->at[1],camera->at[2],
			0,0,1));
	} else 
#endif
	{
		GLCHK(glScalef(.5f,0.5f,.5f));
		GLCHK(glTranslatef(0.0f, -15.0f, -2.50f));
		GLCHK(glRotatef(-90,1.0f,0.0f,0.0f));
		GLCHK(glRotatef(45*(viewmode-4),0.0f,0.0f,1.0f));

		// Real view transform
		cameraz+=16;
		if(cameraz>200) cameraz=-200;
		zoom=viewmode/5.0f;
		GLCHK(glLoadIdentity());
		GLCHK(gluLookAt(50*zoom,20*zoom,cameraz,0,0,0,0,1,0));
	}
	
	GLCHK(glShadeModel(GL_SMOOTH));

	GLCHK(glClear(GL_DEPTH_BUFFER_BIT));


	GLCHK(glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE));	// Handy pre-lighting
	GLCHK(glEnable(GL_BLEND));
	GLCHK(glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA));

	static int currFrame=0;
#if 1
//printf("Displaying scene\n");	
	if(fishFight.sceneEntity) {
		GLCHK(glPushMatrix());
		GLCHK(glScalef(9.0f,9.0f,9.0f));
		std::vector<Frame *>::iterator p;
		for(p=fishFight.sceneEntity->frame.begin();p!=fishFight.sceneEntity->frame.end();p++) {
			Frame *f=*p;
			display_frame(f);
		}
		GLCHK(glPopMatrix());
	}
#endif
	
//printf("Displaying model\n");	
#if 0
	GLCHK(glPushMatrix());
	GLCHK(glMultMatrixf(fishFight.getHero()->getTransform()));
	display_frame(fishFight.getHero()->getCurrLowerFrame());
	GLCHK(glPushMatrix());
	GLCHK(glMultMatrixf(fishFight.getHero()->getCurrLowerTransform()));
	display_frame(fishFight.getHero()->getCurrUpperFrame());
	GLCHK(glPopMatrix());
	GLCHK(glPopMatrix());

	float *t=fishFight.getHero()->getTransform();
	printf("Hero pos: %.2f,%.2f,%.2f\n",t[12],t[13],t[14]);	
#endif


	std::vector<Piece *>::iterator p;
	for(p=fishFight.piece.begin();p!=fishFight.piece.end();p++) {
		Piece *piece=*p;
		GLCHK(glPushMatrix());
		GLCHK(glMultMatrixf(piece->getTransform())); 
		display_entity(piece->getModel());
		GLCHK(glPopMatrix());
	}

	fishFight.updateLevel(timeGetTime());		// Windowsism

//printf("Done display\n");	
	glutSwapBuffers();
	glutPostRedisplay();
//printf("Exit display\n");	
}

float scale=4.0f;

static
void keydown (unsigned char key, int x, int y)
{
	printf("Got keydown event\n");
	switch (key) {
	case 'a':
		fishFight.buttonEvent(FF_LEFT,1,timeGetTime());
		fishFight.buttonEvent(FF_SQUARE,1,timeGetTime());
		break;
	case 's':
		fishFight.buttonEvent(FF_UP,1,timeGetTime());
		break;
	case 'z':
		fishFight.buttonEvent(FF_DOWN,1,timeGetTime());
		fishFight.buttonEvent(FF_CROSS,1,timeGetTime());
		break;
	case 'd':
		fishFight.buttonEvent(FF_RIGHT,1,timeGetTime());
		fishFight.buttonEvent(FF_CIRCLE,1,timeGetTime());
		break;
	case '\n':
	case 'w':
		fishFight.buttonEvent(FF_CROSS,1,timeGetTime());
		break;
	case 27:
	case 'x':			/* cross button */
		exit(0);
		break;
	case '0':
	case '1':
	case '2':
	case '3':
	case '4':
	case '5':
	case '6':
	case '7':
		viewmode=key-'0';
		break;
	case '.': freeze=1-freeze; break;
	case ' ':
		freeze=1;
		step=1;
		break;
	default:
		;
	}
}


static
void keyup (unsigned char key, int x, int y)
{
	printf("Got keyup event\n");
	switch (key) {
	case 'a':
		fishFight.buttonEvent(FF_LEFT,0,timeGetTime());
		break;
	case 's':
		fishFight.buttonEvent(FF_UP,0,timeGetTime());
		break;
	case 'z':
		fishFight.buttonEvent(FF_DOWN,0,timeGetTime());
		break;
	case 'd':
		fishFight.buttonEvent(FF_RIGHT,0,timeGetTime());
		break;
	case 'w':
		fishFight.buttonEvent(FF_CROSS,0,timeGetTime());
		break;
	default:
		;
	}
}

static
void specialdown (int key, int x, int y)
{
	printf("Got specialdown event\n");
	switch (key) {
	case GLUT_KEY_LEFT:
		fishFight.buttonEvent(FF_LEFT,1,timeGetTime());
		break;
	case GLUT_KEY_UP:
		fishFight.buttonEvent(FF_UP,1,timeGetTime());
		break;
	case GLUT_KEY_DOWN:
		fishFight.buttonEvent(FF_DOWN,1,timeGetTime());
		break;
	case GLUT_KEY_RIGHT:
		fishFight.buttonEvent(FF_RIGHT,1,timeGetTime());
		break;
	default:
		;
	}
}


static
void specialup (int key, int x, int y)
{
	printf("Got specialup event\n");
	switch (key) {
	case GLUT_KEY_LEFT:
		fishFight.buttonEvent(FF_LEFT,0,timeGetTime());
		break;
	case GLUT_KEY_UP:
		fishFight.buttonEvent(FF_UP,0,timeGetTime());
		break;
	case GLUT_KEY_DOWN:
		fishFight.buttonEvent(FF_DOWN,0,timeGetTime());
		break;
	case GLUT_KEY_RIGHT:
		fishFight.buttonEvent(FF_RIGHT,0,timeGetTime());
		break;
	default:
		;
	}
}

static
void joystick (unsigned int buttonMask, int x, int y, int z)
{
	GLCHK(glClearColor(x * 1.0f/2000.0f + 0.5f, y * 1.0f/2000.0f + 0.5f, 1.0f, 1.0f));
}

void playSound(int id)
{
}

int main(int argc, char* argv[])
{
printf("Started.\n");
	fishFight.newGame();
	fishFight.load(1);
printf("New game initialized\n");
	glutInit(&argc, argv);
        glutInitDisplayMode(GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
        glutInitWindowSize(480, 272);
	glutCreateWindow( __FILE__ );
	glutKeyboardFunc(keydown);
	glutKeyboardUpFunc(keyup);
    glutSpecialFunc(specialdown);
    glutSpecialUpFunc(specialup);
	glutJoystickFunc(joystick, 0);
printf("Generating textures\n");
	GLCHK(glGenTextures(NTEX, texid));	// Hard coded.
#ifndef SURVIVALHORROR
	upload_textures(fishFight.getSceneEntity());
#else
	upload_textures(&fishFight);
#endif
	std::vector<Piece *>::iterator p;
	for(p=fishFight.piece.begin();p!=fishFight.piece.end();p++) {
		Piece *piece=*p;
		upload_textures(piece->getModel());
	}
	
	glutReshapeFunc(reshape);
	glutDisplayFunc(display);
printf("Started.\n");

	GLCHK(glFrontFace(GL_CW));
	GLCHK(glEnable(GL_DEPTH_TEST));
	GLCHK(glEnable(GL_CULL_FACE));
	GLCHK(glEnable(GL_TEXTURE_2D));

	glutMainLoop();
	return 0;
}

